from ReadOnlyDatabase import ReadOnlyDatabase
import shutil
import os.path
import Queue
from Action import Action
from Property import Property
from Sentence import Sentence

class Database(ReadOnlyDatabase):
    
    
    
    #Pre: Silent is a True or False boolean
    #Post: Constructs the database object 
    #comment: All database read operations come from the ReadOnlyDatabase class 
    def __init__(self,silent=False):
        ReadOnlyDatabase.__init__(self,silent)
        
    
    #Pre: filename is a String pointing to a valid file location
    #Post: Creates a new database file and fills in the required tables and properties
    #comment:  
    def createNewDatabase(self,filename):
                
        sentenceTable = """create table IF NOT EXISTS Sentence 
                            (id INTEGER PRIMARY KEY,sentence TEXT)"""
        linkTable = """create table IF NOT EXISTS Link
                        (reply_to_sentence INTEGER NOT NULL, reply_sentence INTEGER NOT NULL, ordering INTEGER DEFAULT -1, PRIMARY KEY (reply_to_sentence, reply_sentence))"""
        propertyTable = """create table IF NOT EXISTS Property
                        (id INTEGER PRIMARY KEY, name TEXT NOT NULL, show INTEGER DEFAULT 1)"""
        propertyListTable = """create table IF NOT EXISTS Property_list
                        (property_id INTEGER NOT NULL, sentence_id INTEGER NOT NULL,value TEXT, PRIMARY KEY (property_id, sentence_id))"""
        actionTable = """create table IF NOT EXISTS Action
                        (id INTEGER PRIMARY KEY, action TEXT NOT NULL)"""
        actionListTable = """create table IF NOT EXISTS Action_list
                        (action_id INTEGER NOT NULL, sentence_id INTEGER NOT NULL, ordering INTEGER DEFAULT -1, kind INTEGER NOT NULL, PRIMARY KEY (action_id, sentence_id, kind))"""
        
        self.openDatabase(filename)
        
        if not self.cursor == None:
            #fill database tables
            cursor = self.cursor
            cursor.execute(sentenceTable)
            cursor.execute(linkTable)
            cursor.execute(propertyTable)
            cursor.execute(propertyListTable)
            cursor.execute(actionTable)
            cursor.execute(actionListTable)
            
            self.connection.commit()
            
            #Add default properties
            self.addNewProperty("is_root", self.ENABLED)
            self.addNewProperty("audio_url",self.ENABLED)
            self.addNewProperty("avatar_type", self.ENABLED)
            
        else:
            self.printMessage("No database connection available")
        
        

    #Pre: filename (optional) is a String representing a valid file location, overwite is a Boolean
    #     if set on True the given filename will be overwriten if it already exists
    #     use with care!
    #Post: creates a file copy of the "original" database
    #comment: If filename is None the backup will be in the same directory of the 
    #         original database with a filename+timestamp filename 
    def backUpDatabase(self,filename=None,overwrite=False):
        if filename == None:
            databaseUrl = self.filename
        
            #check if file exist
            if not os.path.exists(databaseUrl) or not os.path.isfile(databaseUrl):
                #create file
                file = open(databaseUrl,"w")
                file.close()

        
            #make backup 
            date = datetime.now()
            dtuple = date.timetuple()
            backupUrl = self.name+" "+str(dtuple[2])+"-"+str(dtuple[1])+"-"+str(dtuple[0])+" "+str(dtuple[3])+"_"+str(dtuple[4])+".bak"
            shutil.copy(databaseUrl, backupUrl)
        else:
            if not os.path.exists(filename) or overwrite:
                shutil.copy(self.filename, filename)
            else:
                self.printMessage("Backup FAILED, backup file already existed: "+filename)
            
            
    
    #Pre: sentenceObject is a Sentence object filled with at least a sentence String
    #     and NO !! id !!
    #Post: adds a new sentence to the database with all information out of the sentenceObject
    #comment: Use updateSentence if your not 100% sure that the sentence doesn't already exist in the
    #         database
    def addNewSentence(self,sentenceObject,peek=False,shownPropertysOnly=True):
        if not sentenceObject == None and sentenceObject.getId() == None:
            
            if not self.cursor == None and not self.connection == None:
                
                #Add sentence String (create new sentence row)
                insert = "INSERT INTO Sentence (sentence) VALUES (?)"
                
                self.cursor.execute(insert,(sentenceObject.getSentence(),))
                self.connection.commit()
                
                sentenceId = self.cursor.lastrowid
                
                #add sentence links
                links = sentenceObject.getChildren()
                #No children links should already exist in this sentenceObject
                for link in links:
                    self.addNewLink(link)
                    
                links = sentenceObject.getParents()
                
                for link in links:
                    link.fillInReplyId(sentenceId)
                    self.addNewLink(link)
                
                #add sentence actions
                
                actions = sentenceObject.getPreActionList()
                
                for action in actions:
                    self.updateActionList(action,sentenceId)
                    
                actions = sentenceObject.getPostActionList()
                
                for action in actions:
                    self.updateActionList(action,sentenceId)
                                
                #add sentence property's
                propertyDict = sentenceObject.getPropertyDict()
                keys = propertyDict.keys()
                for key in keys:
                    self.updatePropertyList(propertyDict[key],sentenceId)
                
                
                
                return self.getSentenceById(sentenceId,peek,shownPropertysOnly) 
            else:
                self.printMessage("No database connection available to add sentence") 
        else:
            self.printMessage("Trying to add a None sentence to database or sentence with an Id: "+sentenceObject)
    
    #Pre: sentenceObject is a Sentence object filled with at least a sentence String
    #Post:updates all information in the database with the sentenceObject. If sentence has
    #     no id then the function addNewSentence is called
    #     returns the updated sentence object
    #comment: Updates all information overriding information already in the database for
    #         this sentence (id) could slow down the system with slow file I/O
    #         Can outdate Property lists, Action list 
    def updateSentence(self,sentenceObject,peek=False,shownPropertysOnly=True):
        if not sentenceObject == None:
            if sentenceObject.getId() == None:
                return self.addNewSentence(sentenceObject,peek,shownPropertysOnly)
            else:
                if not self.cursor == None and not self.connection == None:
                #update Sentence
                    select = "SELECT * FROM Sentence WHERE id = ?"
                    self.cursor.execute(select,(sentenceObject.getId(),))
                    
                    row = self.cursor.fetchall()
                    if len(row) == 1:
                        if sentenceObject.getSentence() == row[0]['sentence']:
                            #No update necessary
                            pass
                        else:
                            update = "UPDATE Sentence SET sentence = ? WHERE id = ?"
                            self.cursor.execute(update,(sentenceObject.getSentence(),sentenceObject.getId()))
                            self.connection.commit()
                            
                    else:
                        if len(row) == 0:
                            self.printMessage("No sentence found with id: "+sentenceObject.getId())
                        else:
                            self.printMessage("Multiple sentences found with id: "+sentenceObject.getId())
                    
                    #update Link
                    links = sentenceObject.getChildren()
                    for link in links:
                        self.updateLink(link)
                    
                    links = sentenceObject.getParents()
                    for link in links:
                        self.updateLink(link)
                        
                    #update Action_list
                    actions = sentenceObject.getPreActionList()
                    for action in actions:
                        self.updateActionList(action,sentenceObject.getId())
                        
                    actions = sentenceObject.getPostActionList()
                    for action in actions:
                        self.updateActionList(action,sentenceObject.getId())
                    
                    #update Property_List
                    propertyDict = sentenceObject.getPropertyDict()
                    keys = propertyDict.keys()
                    for key in keys:
                        property = propertyDict[key]
                        self.updatePropertyList(property,sentenceObject.getId())
                    
                    return self.getSentenceById(sentenceObject.getId(),peek,shownPropertysOnly)
                else:
                    self.printMessage("No database connection available")
                    return sentenceObject 
        else:
            self.printMessage("Trying to update a None sentenceObject: "+sentenceObject)
            return sentenceObject
    #Pre: link is a Link object
    #Post: Adds a new Link object in the database
    #comment:
    def addNewLink(self,link):
        if not link == None:
            
            if not link.getReplyId() == None:
                if not self.doesLinkExist(link.getReplyToId(), link.getReplyId()):
                    if not self.cursor == None and not self.connection == None:
                        insert = "INSERT INTO Link (reply_to_sentence,reply_sentence,ordering) VALUES (?,?,?)"
                        self.cursor.execute(insert,(link.getReplyToId(),link.getReplyId(),link.getOrder()))
                        self.connection.commit()
                    else:
                        self.printMessage("No database connection available")
                else:
                    self.printMessage("Anomaly link was already in the database while it should not be: "+link)
            else:
                self.printMessage("Trying to add a incomplete Link object :"+link)
                    
            
        else:
            self.printMessage("Trying to add a None link object")
    
    #Pre: link is a Link object
    #Post: Updates the values in the Link table
    #comment:
    def updateLink(self,link):
        if not link == None:
            #check if link already exist in the database
            if not self.doesLinkExist(link.getReplyToId(), link.getReplyId()):
                #add new link in database
                self.addNewLink(link)
            else:
                #update the link values in database
                if not self.cursor == None and not self.connection == None:
                    update = "UPDATE Link SET ordering = ? WHERE reply_to_sentence = ? AND reply_sentence = ?"
                    self.cursor.execute(update,(link.getOrder(),link.getReplyToId(),link.getReplyId()))
                    self.connection.commit()
                else:
                    self.printMessage("No database connection available")
        else:
            self.printMessage("Trying to update a None link object")
        
    #Pre: parentId, childId are Integers from existing sentences 
    #Post: Removes the Link row representing the link between parentId and childId
    #comment: The childId sentence could break lose and float unreferenced in the
    #         database. (for later use?)
    def removeChildFrom(self,parentId,childId):
        if parentId == None or childId == None:
            self.printMessage("ParentId ("+str(parentId)+" or childId ("+str(childId)+") is not given")
        else:
            if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Link WHERE reply_to_sentence = ? AND reply_sentence = ?"
                self.cursor.execute(delete,(parentId,childId))
            else:
                self.printMessage("No database connection available") 
    #Pre:
    #Post: removes all row from Link with reply_to or reply referencing to a none existing sentence
    #comment: (This should never be the case)
    def cleanLink(self):
        if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Link WHERE reply_to_sentence NOT IN (SELECT id FROM Sentence) OR reply_sentence NOT IN (SELECT id FROM Sentence) "
                self.cursor.execute(delete)
                self.connection.commit()
        else:
            self.printMessage("No database connection available") 
    
    #Pre: sentenceId and actionId are Integers from existing sentence or action id's and kind if given is POSTACTION or PREACTION
    #Post: Removes the Action row(s) from Action_list representing the Action from sentenceId with actionId
    #comment: The Action it self is not removed only the link with this sentenceId 
    #         If no kind is given both PREACTION and POSTACTION are removed if having the same actionId
    def removeActionFrom(self,sentenceId,actionId,kind=-1):
        if not self.cursor == None and not self.connection == None:
            delete = "DELETE FROM Action_list WHERE sentence_id = ? AND action_id = ?"
            if not kind == -1 and (kind == Action.PREACTION or kind ==  Action.POSTACTION):
                delete += " AND kind = ?"
                self.cursor.execute(delete,(sentenceId,actionId,kind))
            else:
                self.cursor.execute(delete,(sentenceId,actionId))
            self.connection.commit()
        else:
            self.printMessage("No database connection available")
    
    #Pre: sentenceId and propertyId are Integers from existing sentence or property id's
    #Post: Removes the Property row from Property_list representing the Property from sentenceId with propertyId
    #comment: The Property it self is not removed only the link with this sentenceId  
    def removePropertyFrom(self,sentenceId,propertyId):
        if not self.cursor == None and not self.connection == None:
            delete = "DELETE FROM Property_list WHERE sentence_id = ? AND property_id = ?"
            self.cursor.execute(delete,(sentenceId,propertyId))
            self.connection.commit()
        else:
            self.printMessage("No database connection available")
    
    #Pre: sentenceId is a Integer from a existing sentence
    #Post: Removes the rows from Sentence, Link (reply_to_sentence or reply_sentence == sentence_id),
    #      Action_list, Property_list with sentenceId
    #comment: Child of this sentence could break lose and float unreferenced in the database
    #         Property or Action referenced by this sentence is not removed
    def removeSentence(self,sentenceId):
        if not self.cursor == None and not self.connection == None:
            if not sentenceId == None:
                 
                 if self.doesSentenceExistById(sentenceId):
                     #remove sentence
                     delete = "DELETE FROM Sentence WHERE id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                     
                     #remove links
                     delete = "DELETE FROM Link WHERE reply_sentence = ? OR reply_to_sentence = ?"
                     self.cursor.execute(delete,(sentenceId,sentenceId))
                     self.connection.commit()
                                          
                     #remove actions
                     delete = "DELETE FROM Action_list WHERE sentence_id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                                          
                     #remove propertys
                     delete = "DELETE FROM Property_list WHERE sentence_id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                     
                 else:
                     self.printMessage("Trying to remove a none existing sentence: "+str(sentenceId))
            else:
                self.printMessage("Trying to remove a None sentenceId: "+str(sentenceId))
        else:
            self.printMessage("No database connection available")
    
    #Pre: sentence is a Integer from a existing sentence
    #Post: uses removeSentence on sentenceId and all children that become orphaned
    #comment: Use with care!! using this on root sentence could empty the whole database
    #         Orphan detection is crude loops in the orphaned string of sentences could leave them floating in the database
    def removeSentenceWithoutFloat(self,sentenceId):
        if not self.cursor == None and not self.connection == None:
            if not sentenceId == None:
                 if self.doesSentenceExistById(sentenceId):
                     #remove sentence
                     delete = "DELETE FROM Sentence WHERE id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                     
                     #get possible orphans
                     select = "SELECT reply_sentence FROM Link WHERE reply_to_sentence = ?"
                     self.cursor.execute(select,(sentenceId,))
                     possibleOrphans = []
                     rows = self.cursor.fetchall()
                     for row in rows:
                         possibleOrphans.append(row['reply_sentence'])  
                     
                     #remove links
                     delete = "DELETE FROM Link WHERE reply_sentence = ? OR reply_to_sentence = ?"
                     self.cursor.execute(delete,(sentenceId,sentenceId))
                     self.connection.commit()
                                          
                     #remove actions
                     delete = "DELETE FROM Action_list WHERE sentence_id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                                          
                     #remove propertys
                     delete = "DELETE FROM Property_list WHERE sentence_id = ?"
                     self.cursor.execute(delete,(sentenceId,))
                     self.connection.commit()
                     
                     #remove orphans
                     for item in possibleOrphans:
                         select = "SELECT * FROM Link WHERE reply_sentence = ?"
                         self.cursor.execute(select,(item,))
                         connections = self.cursor.fetchall()
                         if len(connections) == 0:
                             #sentence is orphaned remove it
                             if self.doesSentenceExistById(item):
                                 self.removeSentenceWithoutFloat(item)
                             
                     
                 else:
                     self.printMessage("Trying to remove a none existing sentence: "+str(sentenceId))
            else:
                self.printMessage("Trying to remove a None sentenceId: "+str(sentenceId))
        else:
            self.printMessage("No database connection available")
    
    #Pre:
    #Post: RemoveSentence() on any sentenceId not linked (indirectly) with a sentence with 
    #      property is_root == 1
    #comment: Use with care!! if no root is set everything could be removed from the database
    def removeAllOrphans(self):
        if not self.cursor == None and not self.connection == None:
            #get all sentence Id's
            select = "SELECT id FROM Sentence"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            completeList = []
            for row in rows:
                completeList.append(int(row['id']))
            
            queue = Queue.Queue()
            alreadyDone = []
            
            #get root
            roots = self.getRootSentences()
            for root in roots:
                queue.put(root.getId())
                
            
            #remove all sentences connected to the sentences in the queue 
            #remove sentence from queue and completeList add to already done list
            #add connected sentences to the queue
            while not queue.empty():
                sentenceId = queue.get()
                
                try:
                    alreadyDone.index(sentenceId)
                    
                except:
                    completeList.remove(sentenceId)
                    alreadyDone.append(sentenceId)
                    select = "SELECT reply_sentence FROM Link WHERE reply_to_sentence = ?"
                    self.cursor.execute(select,(sentenceId,))
                    rows = self.cursor.fetchall()
                    for row in rows:
                        queue.put(int(row['reply_sentence']))
                        
        
            
            #remove remaining sentence in complete list
            for item in completeList:
                self.removeSentence(item)
        else:
            self.printMessage("No database connection available")
    
    #Pre: actionId is a Integer from a existing Action
    #Post: Removes the row from Action with actionId and any row in Action_list with actionId
    #comment: If you only want to change the action String use updateAction 
    def removeAction(self,actionId):
        if not self.cursor == None and not self.connection == None:
            delete = "DELETE FROM Action WHERE id = ?"
            self.cursor.execute(delete,(actionId,))
            self.connection.commit()
            
            delete = "DELETE FROM Action_List WHERE action_id = ?"
            self.cursor.execute(delete,(actionId,))
            self.connection.commit()
            
        else:
            self.printMessage("No database connection available")
    
    #Pre: actionId is a Integer from a existing Action, action is a none empty String
    #Post: updates the action String of Action with actionId 
    #comment: 
    def updateAction(self,actionId, action):
        if not self.cursor == None and not self.connection == None:
            
            if not actionId == None and not action == None and len(action) > 0:
                update = "REPLACE INTO Action (id, action) VALUES (?,?)"
                self.cursor.execute(update,(actionId,action))
                self.connection.commit()
            else:
                self.printMessage("Updating Action with None id or None action: "+str(actionId)+" | "+action)           
            
        else:
            self.printMessage("No database connection available")
    
    #Pre: action is a none empty String
    #Post: Adds a new Action to the Action table and returns it's id
    #comment: Better leave this to the database class
    def addNewAction(self,action):
        if not action == None and len(action) > 0:
            if not self.doesActionExist(action):
                insert = "INSERT INTO Action (action) VALUES (?)"
                if not self.cursor == None and not self.connection == None:
                    self.cursor.execute(insert,(action,))
                    self.connection.commit()
                    return self.cursor.lastrowid
                else:
                    self.printMessage("No database connection available")
                    return None    
            else:
                select = "SELECT id FROM Action WHERE action = ?"
                if not self.cursor == None and not self.connection == None:
                    self.cursor.execute(select,(action,))
                    try:
                        return int(self.cursor.fetchone()['id'])
                    except:
                        self.printMessage("Failed to read the id of action: "+str(action))
                        return None
                else:
                    self.printMessage("No database connection available")
                    return None 
        else:
            self.printMessage("Trying to add a empty action: "+str(action))
            return None
    
    #Pre: action is a Action object, sentenceId is a Integer of a existing sentence
    #Post: action in Action_list is updated or added if not existing yet
    #comment: If action does not yet exist it's also created new
    def updateActionList(self,action,sentenceId):
        if not action == None:
            actionId = action.getId()
            if actionId == None:
                actionId = self.addNewAction(action.getAction())
            else:
                self.updateAction(actionId, action.getAction())
                
            if self.doesSentenceExistById(sentenceId):
                if not self.cursor == None and not self.connection == None:
                    if not actionId == None:
                        replace = "REPLACE INTO Action_list (action_id,sentence_id,ordering,kind) VALUES (?,?,?,?)"
                        self.cursor.execute(replace,(actionId,sentenceId,action.getOrder(),action.getKind()))
                        self.connection.commit()
                    else:
                        self.printMessage("Action id is not found: "+str(action))
                else:
                    self.printMessage("No database connection available")
            else:
                self.printMessage("Trying to add a Action to a none existing sentence: "+sentenceId+" | "+str(action))
            
            
                
        else:
            self.printMessage("Trying to update a None action object: "+str(action))
    
    #Pre:
    #Post: removes all rows from Action_List referencing to none existing sentence or actions
    #comment: (this should never be the case)
    def cleanActionList(self):
        if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Action_list WHERE sentence_id NOT IN (SELECT id FROM Sentence) OR action_id NOT IN (SELECT id FROM Action) "
                self.cursor.execute(delete)
                self.connection.commit()
        else:
            self.printMessage("No database connection available") 
    
    #Pre:
    #Post: removes all rows from Action with ActionId that is not used in Action_list
    #comment:
    def cleanAction(self):
        if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Action WHERE id NOT IN (SELECT action_id FROM Action_list)"
                self.cursor.execute(delete)
                self.connection.commit()
        else:
            self.printMessage("No database connection available") 
    
    #Pre: propertyId is a Integer from a existing Property
    #Post: removes the row from Property with propertyId and any row in Property_List with propertyID
    #comment: If you only want to change the property name or show use updateProperty orshowProperty 
    def removeProperty(self,propertyId):
        if not self.cursor == None and not self.connection == None:
            delete = "DELETE FROM Property WHERE id = ?"
            self.cursor.execute(delete,(propertyId,))
            self.connection.commit()
            
            delete = "DELETE FROM Property_List WHERE property_id = ?"
            self.cursor.execute(delete,(propertyId,))
            self.connection.commit()
            
        else:
            self.printMessage("No database connection available")
    
    #Pre: propertyId is a Integer from a existing Property, name is a none empty String and show = ENABLE or DISABLE
    #Post: updates the property name and show value from propertyId 
    #comment: if you only want to disable or enable show use showProperty
    def updateProperty(self,propertyId,name,show):
        if not self.cursor == None and not self.connection == None:
            
            if not propertyId == None and not name == None and len(name) > 0 and (show == self.ENABLED or show == self.DISABLED):
                update = "REPLACE INTO Property (id, name, show) VALUES (?,?,?)"
                self.cursor.execute(update,(propertyId,name,show))
                self.connection.commit()
            else:
                self.printMessage("Updating Property with None id or None name or incorrect show: "+str(propertyId)+" | "+name+" |" +str(show))           
            
        else:
            self.printMessage("No database connection available")
    
    #Pre: property is a PropertyObject and sentenceId is a existing sentenceId
    #Post: property is Property_list is updated or added if not already existing
    #comment: If property doesn't yet exist it's also added new with show on ENABLED
    def updatePropertyList(self,property,sentenceId):
        if not property == None:
            propertyId = property.getId()
            if propertyId == None:
                propertyId = self.addNewProperty(property.getName(),self.ENABLED)
            
            if self.doesSentenceExistById(sentenceId):
                if not self.cursor == None and not self.connection == None:
                    if not propertyId == None:
                        replace = "REPLACE INTO Property_list VALUES (?,?,?)"
                        self.cursor.execute(replace,(propertyId,sentenceId,property.getValue()))
                        self.connection.commit()
                    else:
                        self.printMessage("Property id not found: "+str(property))
                else:
                    self.printMessage("No database connection available")
            else:
                self.printMessage("Trying to add a Property to a none existing sentence: "+sentenceId+" | "+str(property))
            
            
                
        else:
            self.printMessage("Trying to update a None property object: "+str(property))
    
    #Pre: name is a none empty String and show is ENABLE or DISABLE
    #Post: Adds a new Property to the Property table
    #comment: Better leave this to the database class
    def addNewProperty(self,name,show):
        if not name == None and len(name) > 0 and (show == self.ENABLED or show == self.DISABLED):
            
            if not self.doesPropertyExist(name):
            
                insert = "INSERT INTO Property (name,show) VALUES (?,?)"
                if not self.cursor == None and not self.connection == None:
                    self.cursor.execute(insert,(name,show))
                    self.connection.commit()
                    
                    return self.cursor.lastrowid
                else:
                    self.printMessage("No database connection available")
                    return None
            
            else:
                select = "SELECT id FROM Property WHERE name = ?"
                if not self.cursor == None and not self.connection == None:
                    self.cursor.execute(select,(name,))
                    try:
                        return int(self.cursor.fetchone()['id'])
                    except:
                        self.printMessage("Failed to read Property id from the database: "+name)
                        return None
                else:
                    self.printMessage("No database connection available")
                
                return None
        else:
            self.printMessage("trying to add invalid property: "+name+" "+str(show))
            return None
    #Pre:
    #Post: removes all rows from Property_list referencing to none existing sentences or property
    #comment: (This should never be the case)
    def cleanPropertyList(self):
        if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Property_list WHERE sentence_id NOT IN (SELECT id FROM Sentence) OR property_id NOT IN (SELECT id FROM Property) "
                self.cursor.execute(delete)
                self.connection.commit()
        else:
            self.printMessage("No database connection available")
    
    #Pre:
    #Post: removes all rows from Property with propertyId that is not used in Property_list
    #comment: 
    def cleanProperty(self):
        if not self.cursor == None and not self.connection == None:
                delete = "DELETE FROM Property WHERE id NOT IN (SELECT property_id FROM Property_list)"
                self.cursor.execute(delete)
                self.connection.commit()
        else:
            self.printMessage("No database connection available")
    
    #Pre: propertyId is a Integer from a existing Property and show is ENABLE or DISABLE
    #Post: updates the show value of row in Property with propertyId
    #comment:  
    def showProperty(self,propertyId,show):
        if show == self.DISABLED or show == self.ENABLED:
            if not self.cursor == None and not self.connection == None:
                if self.doesPropertyExistId(propertyId):
                    update =  "UPDATE Property SET show = ? WHERE id = ?"
                    self.cursor.execute(update,(show,propertyId))
                    self.connection.commit()  
                else:
                    self.printMessage("Trying to update property show value on none existing property: "+str(propertyId))
            else:
                self.printMessage("No database connection available")
            

        
    