
from Action import Action
from Property import Property
from Sentence import Sentence
from Link import Link
import sqlite3

class ReadOnlyDatabase():
    
    DISABLED = 0
    ENABLED = 1
    
    #Pre: silent is a boolean False or True
    #Post: constructs a ReadOnlyDatabase
    #comment:  
    def __init__(self,silent=False):
        
        self.silent = silent
        self.reset()
        
                
    #Pre: filename is a valid and existing filename
    #Post: opens and loads in the database from filename
    #comment: Does not throw a exception when loading fails!  
    def openDatabase(self,filename):
        try:
            self.connection = conn = sqlite3.connect(filename)
            conn.row_factory = sqlite3.Row
            conn.text_factory = str
            self.cursor = conn.cursor()
            
            self.filename = filename
        except:
            self.printMessage("Database failed to connect to: "+filename)
            
        
    
    #Pre: A database is loaded
    #Post: closes the loaded database
    #comment:      
    def closeDatabase(self):
        try:
            self.cursor.close()
            self.cursor = None
        except:
            pass        
        
    
    #Pre: id is a Integer, peek is a boolean, shownPropertysOnly is a boolean
    #Post: returns a Sentence object or None, If peek = True all Link objects have there peek value filled in
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment:          
    def getSentenceById(self,id,peek=False,shownPropertysOnly=True):
        
        #Check if database is connected
        if not self.cursor == None:
            #Get sentence if None return 
            select = "SELECT * FROM Sentence WHERE id = ?"
            self.cursor.execute(select,(id,))
            rows = self.cursor.fetchall()
            
            if len(rows) > 0:
                if len(rows) == 1:
                    
                    sentence = Sentence(rows[0]['sentence'],id)
                    
                    #Get all Child links
                    select = "SELECT * FROM Link WHERE reply_to_sentence = ? ORDER BY ordering ASC"
                    self.cursor.execute(select,(id,))
                    
                    rows = self.cursor.fetchall()
                    for row in rows:
                        link = Link(id,int(row['reply_sentence']),int(row['ordering']))
                        
                        if peek:
                            select = "SELECT * FROM Sentence WHERE id = ?"
                            self.cursor.execute(select,(link.getReplyId(),))
                            peeks = self.cursor.fetchall()
                            if len(peeks) == 1:
                                link.addReplyPeek(peeks[0]['sentence'])
                                link.addReplyToPeek(sentence.getSentence())
                            else:
                                self.printMessage("Failed to fill in peek for sentence: "+str(id)+" - "+str(link.getReplyId()))
                        
                        sentence.addChild(link)
                    
                    #Get all Parent links
                    select = "SELECT * FROM Link WHERE reply_sentence = ?"
                    self.cursor.execute(select,(id,))
                    
                    rows = self.cursor.fetchall()
                    for row in rows:
                        link = Link(int(row['reply_to_sentence']),id,int(row['ordering']))
                        
                        if peek:
                            select = "SELECT * FROM Sentence WHERE id = ?"
                            self.cursor.execute(select,(link.getReplyToId(),))
                            peeks = self.cursor.fetchall()
                            if len(peeks) == 1:
                                link.addReplyToPeek(peeks[0]['sentence'])
                                link.addReplyPeek(sentence.getSentence())
                            else:
                                self.printMessage("Failed to fill in peek for sentence: "+str(id)+" - "+str(link.getReplyId()))
                        
                        sentence.addParent(link)
                            
                    #Get all PreActions
                    select = "SELECT * FROM Action_list WHERE sentence_id = ? AND kind = ? ORDER BY ordering ASC"
                    self.cursor.execute(select,(id,Action.PREACTION))
                    rows = self.cursor.fetchall()
                    
                    for row in rows:
                        select = "SELECT * FROM Action WHERE id = ?"
                        self.cursor.execute(select,(row['action_id'],))
                        actionRow = self.cursor.fetchall()
                        if len(actionRow) == 1:
                            action = Action(actionRow[0]['action'],int(row['kind']),int(actionRow[0]['id']),row['ordering'])
                            sentence.addPreAction(action)
                        else:
                            self.printMessage("Anomily detected multiple Actions found with id: "+str(row['action_id']))
                    #Get all PostActions
                    select = "SELECT * FROM Action_list WHERE sentence_id = ? AND kind = ? ORDER BY ordering ASC"
                    self.cursor.execute(select,(id,Action.POSTACTION))
                    rows = self.cursor.fetchall()
                    
                    for row in rows:
                        select = "SELECT * FROM Action WHERE id = ?"
                        self.cursor.execute(select,(row['action_id'],))
                        actionRow = self.cursor.fetchall()
                        if len(actionRow) == 1:
                            action = Action(actionRow[0]['action'],int(row['kind']),int(actionRow[0]['id']),row['ordering'])
                            sentence.addPostAction(action)
                        else:
                            self.printMessage("Anomily detected multiple Actions found with id: "+str(row['action_id']))    
                    
                    #Get all Properties
                    select = "SELECT * FROM Property_list WHERE sentence_id = ?"
                    self.cursor.execute(select,(id,))
                    rows = self.cursor.fetchall()
                    for row in rows:
                        select = "SELECT * FROM Property WHERE id = ?"
                        self.cursor.execute(select,(row['property_id'],))
                        propertyRow = self.cursor.fetchall()
                        if len(propertyRow) == 1:
                            if int(propertyRow[0]['show']) == self.ENABLED or not shownPropertysOnly: 
                                property = Property(propertyRow[0]['name'],row['value'],int(propertyRow[0]['id']),propertyRow[0]['show'])
                                sentence.addProperty(property)
                        else:
                            self.printMessage("Anomily detected multiple Actions found with id: "+str(row['property_id']))    
                    
                        
                    #return filled sentence object
                    return sentence
                
                else:
                    self.printMessage("Anomily detected multiple sentences found with id: "+str(id))
                    return None
            else:
                self.printMessage("Sentence was not found in the database: "+str(id))
                return None
            
            
           
        else:
            self.printMessage("No database connection available")
            return None
    
    #Pre: list is a List with Integers
    #Post: returns a list with Sentence objects or a empty List
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment:
    def getSentencesByIds(self,list,peek=False,shownPropertysOnly=True):
        sentences = []
        for item in list:
            sentence =  self.getSentenceById(item,peek,shownPropertysOnly)
            if not sentence == None:
                sentences.append(sentence)
        
        return sentences
    
    #Pre: propertyName and propertyValue are Strings
    #Post: returns a list of Sentence objects or a empty list if no sentences have 
    #      propertyName and propertyValue
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment: Miss spelled propertyNames or Values return a empty list  
    def getAllSentencesWithPropertyName(self,propertyName,propertyValue,peek=False,shownPropertysOnly=True):
        if len(propertyName) > 0 and len(propertyValue) > 0:
            if not self.cursor == None:
            
                select = "SELECT * FROM Property WHERE name = ?"
                self.cursor.execute(select,(propertyName,))
                rows = self.cursor.fetchall()
                if len(rows) == 1:
                    row = rows[0]
                    select = "SELECT * FROM Property_list WHERE property_id = ? AND value = ?"
                    self.cursor.execute(select,(row['id'],propertyValue))
                    rows = self.cursor.fetchall()
                    sentenceIds = []
                    for row in rows:
                        sentenceIds.append(int(row['sentence_id']))
                    
                    return self.getSentencesByIds(sentenceIds,peek,shownPropertysOnly)
                    
                else:
                    self.printMessage("Multiple Properties found with propertyName: "+propertyName)
                    return []
            
                
            else:
                self.printMessage("No database connection available")
                return []
                
        else:
            self.printMessage("Trying to search for empty property name or value: "+propertyName+" | "+propertyValue)
            return []
        
    #Pre: propertyId is a Integer and propertyValue is a String
    #Post: returns a list of Sentence objects or a empty list if no sentences have 
    #      propertyId and propertyValue
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment: None existing propertyId or miss spelled Values return a empty list  
    def getAllSentencesWithPropertyId(self,propertyId,propertyValue,peek=False,shownPropertysOnly=True):
        
        if not propertyId == None and len(propertyValue) > 0:
            if not self.cursor == None:
                select = "SELECT * FROM Property_list WHERE property_id = ? AND value = ?"
                self.cursor.execute(select,(propertyId,propertyValue))
                rows = self.cursor.fetchall()
                sentenceIds = []
                for row in rows:
                    sentenceIds.append(int(row['sentence_id']))
                
                return self.getSentencesByIds(sentenceIds,peek,shownPropertysOnly)
            else:
                self.printMessage("No database connection available")
                return []
        else:
            self.printMessage("Trying to search for empty property id or value: "+propertyId+" | "+propertyValue)
            return []
    
    #Pre: actionId is a Integer and actionKind PREACTION or POSTACTION
    #Post: returns a list of Sentence objects or a empty list if no sentences have 
    #      actionId and actionKind
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment: None existing actionId returns a empty list
    def getAllSentencesWithAction(self,actionId,actionKind,peek=False,shownPropertysOnly=True):
        if not actionId == None and (actionKind == Action.PREACTION or actionKind == Action.POSTACTION):
            if not self.cursor == None:
                select = "SELECT * FROM Action_list WHERE action_id = ? AND kind = ?"
                self.cursor.execute(select,(actionId,actionKind))
                rows = self.cursor.fetchall()
                sentenceIds = []
                for row in rows:
                    sentenceIds.append(int(row['sentence_id']))
                
                return self.getSentencesByIds(sentenceIds,peek,shownPropertysOnly)
            else:
                self.printMessage("No database connection available")
                return []
        else:
            self.printMessage("Trying to search for empty property id or value: "+propertyId+" | "+propertyValue)
            return []
        
    #Pre:
    #Post: returns all Sentence objects from the database if none exist returns empty list
    #      If shownPropertysOnly equals True only properties with show ENABLED are put in the sentence object
    #      If shownPropertysOnly equals False all linked properties are added to the sentence object
    #comment: Could take a long time
    def getAllSentences(self,peek=False,shownPropertysOnly=True):
        if not self.cursor == None:
            select = "SELECT * FROM Sentence"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            sentenceIds = []
            for row in rows:
                sentenceIds.append(row['id'])
                
            return self.getSentencesByIds(sentenceIds,peek,shownPropertysOnly)
        else:
            self.printMessage("No database connection available")
            return []
    
    
    #Pre:
    #Post: Returns all Action Strings from Action table if none exist returns empty list
    #comment:  
    def getAllActions(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append(row['action'])
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Action Strings with a PREACTION flag in Action_List if none exist returns empty list
    #comment: Defined Actions in Action table can be both PREACTION and POSTACTION  
    def getAllPreActions(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action WHERE id IN (SELECT action_id FROM Action_list WHERE kind = ?)"
            self.cursor.execute(select,(Action.PREACTION,))
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append(row['action'])
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Action Strings with a POSTACTION flag in Action_List if none exist returns empty list
    #comment: Defined Actions in Action table can be both PREACTION and POSTACTION  
    def getAllPostActions(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action WHERE id IN (SELECT action_id FROM Action_list WHERE kind = ?)"
            self.cursor.execute(select,(Action.POSTACTION,))
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append(row['action'])
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre: 
    #Post: returns True if action exists in Action table else returns False 
    #comment:
    def doesActionExist(self,action):
        if not self.cursor == None:
            select = "SELECT * FROM Action WHERE action = ?"
            self.cursor.execute(select,(action,))
            
            return not self.cursor.fetchone() == None
        else:
            self.printMessage("No database connection available")
            return False
    #Pre:
    #Post: Returns all Property names from the Property table with show ENABLED if none exist returns empty list
    #comment:  
    def getAllShownProperties(self):
        if not self.cursor == None:
            select = "SELECT * FROM Property WHERE show = ?"
            self.cursor.execute(select,(self.ENABLED,))
            rows = self.cursor.fetchall()
            propertyStrings = []
            for row in rows:
                propertyStrings.append(row['name'])
    
            return propertyStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Property names from the Property table if none exist returns empty list
    #comment:  
    def getAllProperties(self,shownOnly=True):
        if not self.cursor == None:
            select = "SELECT * FROM Property"
            if shownOnly:
                select = "SELECT * FROM Property WHERE show = 1"
            
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            propertyStrings = []
            for row in rows:
                propertyStrings.append(row['name'])
    
            return propertyStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre: propertyName is a String of a valid Property name
    #Post: returns a list of all values of property with name propertyName
    #comment:
    def getAllPropertyValues(self,propertyName):
        if self.doesPropertyExist(propertyName):
            select = "SELECT id FROM Property WHERE name = ?"
            self.cursor.execute(select,(propertyName,))
            row = self.cursor.fetchone()
            id = row['id']
            select = "SELECT * FROM Property_list WHERE property_id = ?"
            self.cursor.execute(select,(id,))
            
            rows = self.cursor.fetchall()
            list = []
            for row in rows:
                list.append(row['value'])
            
            return list
    
    #Pre:
    #Post: Returns true if property with name exist in Property table else returns false
    #comment:
    def doesPropertyExist(self, name):
        if not self.cursor == None:
            select = "SELECT * FROM Property WHERE name = ?"
            self.cursor.execute(select,(name,))
            
            return not self.cursor.fetchone() == None
        else:
            self.printMessage("No database connection available")
            return False
    
    #Pre:
    #Post: Returns all sentences with Property is_root on 1 if none exist returns a empty list
    #comment:  
    def getRootSentences(self,peek=False,shownPropertysOnly=True):
        return self.getAllSentencesWithPropertyName("is_root","1",peek=False,shownPropertysOnly=True)
    
    #Pre: reply_to (parent) is a Integer and reply (child) is a Integer
    #Post: return True if in Link exist a row with reply_To and reply
    #comment:
    def doesLinkExist(self,reply_to,reply):
        if not self.cursor == None:
            if reply_to == None or reply == None:
                return False
            else:
                select = "SELECT * FROM Link WHERE reply_to_sentence = ?  and reply_sentence = ?"
                self.cursor.execute(select,(reply_to,reply))
                
                return not self.cursor.fetchone() == None
        else:
            self.printMessage("No database connection available")
            return False
        
    #Pre: message is a String
    #Post: prints the message to default out channel if silent is set to False
    #comment: !internal use only!  
    def printMessage(self,message):
        if not self.silent:
            print message
    
    def doesSentenceExistById(self,sentenceId):
        if not self.cursor == None and not self.connection == None:
            select = "SELECT * FROM Sentence WHERE id = ?"
            self.cursor.execute(select,(sentenceId,))
            
            return len(self.cursor.fetchall()) > 0
        
        else:
            self.printMessage("No database connection available")
            return False
        
    def doesPropertyExistId(self,propertyId):
        if not self.cursor == None and not self.connection == None:
            select = "SELECT * FROM Property WHERE id = ?"
            self.cursor.execute(select,(propertyId,))
            
            return len(self.cursor.fetchall()) > 0
        
        else:
            self.printMessage("No database connection available")
            return False
    
    #Pre: propertyName is a string representing a existing property
    #Post: returns the id of the property with propertyName or None if property doesn't exist
    #comment: 
    def getPropertyId(self,propertyName):
        if not propertyName == None and len(propertyName) > 0:
            if self.doesPropertyExist(propertyName):
                select = "SELECT * FROM Property WHERE name = ?"
                self.cursor.execute(select,(propertyName,))
                result = self.cursor.fetchone()
                return int(result['id'])
            else:
                return None
            
    #Pre:
    #Post: Returns all Actions (id, action) from Action table if none exist returns empty list
    #comment:
    def getAllActionsTuple(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append((row['id'],row['action']))
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Pre Actions (id, action) from Action table if none exist returns empty list
    #comment:
    def getAllPreActionsTuple(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action WHERE id IN (SELECT action_id FROM Action_list WHERE kind = 0)"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append((row['id'],row['action']))
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Post Actions (id, action) from Action table if none exist returns empty list
    #comment:
    def getAllPostActionsTuple(self):
        if not self.cursor == None:
            select = "SELECT * FROM Action WHERE id IN (SELECT action_id FROM Action_list WHERE kind = 1)"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            actionStrings = []
            for row in rows:
                actionStrings.append((row['id'],row['action']))
    
            return actionStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns number of sentences with pre action with actionId
    #comment: 
    def getPreActionCount(self,actionId):
        if not self.cursor == None:
            select = "SELECT * FROM Action_list WHERE action_id = ? and kind = 0"
            self.cursor.execute(select,(actionId,))
            rows = self.cursor.fetchall()
            return len(rows)
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns number of sentences with post action with actionId
    #comment:
    def getPostActionCount(self,actionId):
        if not self.cursor == None:
            select = "SELECT * FROM Action_list WHERE action_id = ? and kind = 1"
            self.cursor.execute(select,(actionId,))
            rows = self.cursor.fetchall()
            return len(rows)
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns all Property's (id, name,show) from Action table if none exist returns empty list
    #comment:
    def getAllPropertiesTuple(self,shownOnly=True):
        if not self.cursor == None:
            select = "SELECT * FROM Property"
            if shownOnly:
                select = "SELECT * FROM Property WHERE show = 1"
            self.cursor.execute(select)
            rows = self.cursor.fetchall()
            propertyStrings = []
            for row in rows:
                propertyStrings.append((row['id'],row['name'],row['show']))
    
            return propertyStrings
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns number of sentences with property with propertyId
    #comment:
    def getPropertyCount(self,propertyId):
        if not self.cursor == None:
            select = "SELECT * FROM Property_list WHERE property_id = ?"
            self.cursor.execute(select,(propertyId,))
            rows = self.cursor.fetchall()
            return len(rows)
            
        else:
            self.printMessage("No database connection available")
            return []
    
    #Pre:
    #Post: Returns number of sentences with property with propertyId and propertyValue
    #comment:
    def getPropertyValueCount(self,propertyId,propertyValue):
        if not self.cursor == None:
            select = "SELECT * FROM Property_list WHERE property_id = ? AND value = ?"
            self.cursor.execute(select,(propertyId,propertyValue))
            rows = self.cursor.fetchall()
            return len(rows)
            
        else:
            self.printMessage("No database connection available")
            return []
        
    #Pre:
    #Post: resets all database used variables
    #comment: !Internal use only!
    def reset(self):
        self.closeDatabase()
        self.filename = ""
        self.cursor = None
        self.connection = None
    
    